unit BWTools;

interface

uses Windows, SysUtils, classes;

function IsWinNT:boolean;
function IsWin95:boolean;

function Max(a,b:integer):integer;
function Min(a,b:integer):integer;
function InRange(L,R,x:integer):boolean;
Function RangesIntersect(L1,R1, L2,R2 : integer):boolean;
function Str(i:integer):string;

function CenterStr(Str:string;len:byte):string;
{ Centers string in a line }

function IsBIn(index:integer;storage:byte):boolean;

function CutLeadBlank(s:string):string;

function encode_line(const buf; size:integer):string;

type TDirectMemoryStream = class (TMemoryStream)
       public
         procedure SetPointer(Ptr: Pointer; Size: Longint);
     end;

type TReducedDateTime = record
       Year,
       Month,
       Day,
       DOW,
       Hour,
       Min  : word;
     end;

function DateTimeToReduced(T:TDateTime):TReducedDateTime;
function ReducedToDateTime(T:TReducedDateTime):TDateTime;
function CompareReducedDT(T1,T2:TReducedDateTime):boolean;
// < -1
// = 0
// > +1
Function MakeReducedDT(Year,Month,Day,DOW,Hour,Min : word):TReducedDateTime;

function RunProgram(StartName, Params, StartDir : String):THandle;

implementation

const bin2b64:string='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
      bin2uue:string='`!"#$%&''()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_';

function encode_line(const buf; size:integer):string;
type ta_8u=packed array [0..65530] of byte;
var
  buff: ta_8u absolute buf;
  offset: shortint;
  pos1,pos2: byte;
  i: byte;
  out: string;

begin
  setlength(out, size*4 div 3 + 4);
  fillchar(out[1], size*4 div 3 +2,#0);   (* worst case *)
  offset:=2;
  pos1:=0;
  pos2:=2;
  out[pos2]:=#0;
  while pos1<size do
  begin
    if offset > 0 then
    begin
      out[pos2]:=char(ord(out[pos2]) or ((buff[pos1] and ($3f shl offset)) shr offset));
      offset:=offset-6;
      inc(pos2);
      out[pos2]:=#0;
    end
    else
    if offset < 0 then
    begin
      offset:=abs(offset);
      out[pos2]:=char(ord(out[pos2]) or ((buff[pos1] and ($3f shr offset)) shl offset));
      offset:=8-offset;
      inc(pos1);
    end
    else
    begin
      out[pos2]:=char(ord(out[pos2]) or ((buff[pos1] and $3f)));
      inc(pos2);
      inc(pos1);
      out[pos2]:=#0;
      offset:=2;
    end;
  end;
  if offset=2 then dec(pos2);
  for i:=2 to pos2 do
    out[i]:=bin2uue[ord(out[i])+1];
  encode_line:=copy(out,1,pos2);
end;

  function RunProgram;
  var
      s:string;
      AppName:PChar;
      FParams :PChar;
      CrFlags:DWORD;
      CurDir :PChar;
      StartInfo:TStartupInfo;
      ProcInfo:TProcessInformation;
      res:boolean;

  begin
    if IsWinNt then
    begin
      s:=StartName+#32+Params;
      AppName:=nil;
    end
    else
    begin
      s:=StartName;
      GetMem(AppName,260);
      StrPCopy(AppName,s);
      s:=Params;
    end;
    GetMem(FParams,260);
    StrPCopy(FParams,s);
    GetMem(CurDir,260);
    s:=StartDir;
    StrPCopy(CurDir,s);
    // Set process specific flags
    CrFlags:=NORMAL_PRIORITY_CLASS+CREATE_DEFAULT_ERROR_MODE;
    // set Startup information
    FillMemory(@StartInfo,SizeOf(TStartupInfo),0);
    with StartInfo do
    begin
      cb:=SizeOf(TStartupInfo);
      lpDesktop:=nil;
      lpTitle  :=nil;
      dwFlags  := STARTF_USESHOWWINDOW;
      wShowWindow := SW_SHOW;
    end; // with
    res:=CreateProcess(AppName, FParams, nil, nil, false, CrFlags, nil,
                    CurDir, StartInfo, ProcInfo);
    FreeMem(FParams,260);
    FreeMem(CurDir,260);
    if AppName<>nil then FreeMem(AppName,260);
    result:=0;
    if res then result:=ProcInfo.hProcess;
  end;

procedure TDirectMemoryStream.SetPointer(Ptr: Pointer; Size: Longint);
begin
  inherited;
  Seek(0,soFromBeginning);
end;

Function MakeReducedDT(Year,Month,Day,DOW,Hour,Min : word):TReducedDateTime;
begin
  result.Year:=year;
  result.month:=month;
  Result.Day:=day;
  result.DOW:=dow;
  result.Hour:=hour;
  result.Min:=min;
end;

function CompareReducedDT;
{asm
 push edx
 push esi
 push edi
 push ecx

@label1:
 mov ecx, 6
 mov esi, T1
 mov edi, T2
 cmpsw
 jl @label2
 jg @label3
 xor eax,eax
 jmp @ret1
@label2:
 mov al,$FF
 jmp @ret1
@label3:
 mov al,1
@ret1:
 pop ecx
 pop edi
 pop esi
 pop edx}
begin
  result:=CompareMem(@T1,@T2,SizeOf(TReducedDateTime));
end;

function DateTimeToReduced(T:TDateTime):TReducedDateTime;
var i,j:word;
begin
  DecodeDate(T,Result.Year,Result.Month,Result.Day);
  DecodeTime(T,Result.Hour,Result.Min,i,j);
  Result.DOW:=DayOfWeek(T);
end;

function ReducedToDateTime(T:TReducedDateTime):TDateTime;
begin
  try
    result:=EncodeDate(T.Year,T.Month,T.Day)+EncodeTime(T.Hour,T.Min,0,0);
  except
    on E:EConvertError do
    begin
      Result:=Now;
    end;
  end;
end;

function IsWinNT:boolean;
var VerInfo:TOsVersionInfo;
begin
  VerInfo.dwOsVersionInfoSize:=SizeOf(VerInfo);
  GetVersionEx(VerInfo);
  result:= VerInfo.dwPlatformId = VER_PLATFORM_WIN32_NT;
end;

function IsWin95:boolean;
var VerInfo:TOsVersionInfo;
begin
  VerInfo.dwOsVersionInfoSize:=SizeOf(VerInfo);
  GetVersionEx(VerInfo);
  result:= VerInfo.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS;
end;

function CutLeadBlank;
var i:integer;
begin
  i:=1;
  while s[i]=' ' do inc(i);
  result:=copy(s,i,255);
end;

function IsBIn;
begin
 if ((Storage shr (index-1)) and 1)=0
   then Result:=false else result:=true;
end;

function UpStr(s:string):string;
var i:byte;
    st:string;
begin
 st:=s;
 for i := 1 to Length(s) do st[i] := UpCase(st[i]);
 UpStr:=st;
end;

Function CenterStr(Str:string;Len:byte):string;
var S:String;
    L:byte;

begin
 s:='';
 if len<Length(str) then
 begin
  CenterStr:='';
  Exit;
 end;
 l:=(Len-Length(Str)) div 2;
 while l>0 do
 begin
  s:=s+' ';
  dec(l);
 end;
 for l:=1 to length(Str) do s:=s+str[L];
 CenterStr:=s;
end;

Function Str(i:integer):string;
var s:string;
begin
 System.Str(i,s);
 Str:=s;
end;

function InRange(L,R,x:integer):boolean;
begin
  result:=true;
  if (x<L) or (x>R) then result:=false;
end;

function Max(a,b:integer):integer;
begin
  if a<b then result:=b else result:=a;
end;

function Min(a,b:integer):integer;
begin
  if a>b then result:=b else result:=a;
end;

Function RangesIntersect(L1,R1, L2,R2 : integer):boolean;
begin
  if (Min(L1,R1)>Max(L2,R2)) or (Min(L2,R2)>Max(L1,R1))
     then result:=false
     else result:=true;
end;

end.
