{+---------------------------------------+}
{|                                       |}
{|         Bugsy Wabbit's Header         |}
{|           Additional file             |}
{|              Freeware                 |}
{|         by Eugene Mayevski            |}
{|                                       |}
{|     http://www.ig.com.ua/wabbit       |}
{|       Eugene-Mayevski@usa.net         |}
{+---------------------------------------+}

unit frmSectProp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf, BWHeader, StdCtrls, frmSectEdit;

type
  TBWSectionsPropDlg = class(TForm)
    SecList: TListBox;
    AddBtn: TButton;
    DeleteBtn: TButton;
    UpBtn: TButton;
    DownBtn: TButton;
    LoadBtn: TButton;
    SaveBtn: TButton;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    EditBtn: TButton;
    procedure LoadBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure EditBtnClick(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure SecListClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SecListKeyPress(Sender: TObject; var Key: Char);
  protected
  public
    DSect : TBWHeaderSections;
  end;

  TBWSectionsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue:string; override;
  end;

var
  BWSectionsPropDlg: TBWSectionsPropDlg;

implementation

{$R *.DFM}

procedure TBWSectionsProperty.Edit;
var Editor: TBWSectionsPropDlg;
    j : integer;
begin
  Editor:=nil;
  try
    Editor:=TBWSectionsPropDlg.Create(Application);
    Editor.DSect:=TBWHeaderSections(GetOrdValue);
    for j := 0 to Editor.DSect.Count-1 do    // Iterate
      Editor.SecList.Items.AddObject(Editor.DSect.Item[j].Text,Editor.DSect.Item[j]);
    Editor.ShowModal;
  finally
    Editor.Free;
  end;
end;

function TBWSectionsProperty.GetAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TBWSectionsProperty.GetValue;
begin
  FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;

procedure TBWSectionsPropDlg.LoadBtnClick(Sender: TObject);
var j:integer;
    Form : TCustomForm;
begin
  if not OpenDlg.Execute then exit;
  DSect.LoadFromFile(OpenDlg.FileName);
  SecList.Items.Clear;
  for j := 0 to DSect.Count-1 do    // Iterate
    SecList.Items.AddObject(DSect.Item[j].Text,DSect.Item[j]);
  Form := GetParentForm(DSect.Owner);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
end;

procedure TBWSectionsPropDlg.SaveBtnClick(Sender: TObject);
begin
  if not SaveDlg.Execute then exit;
  DSect.SaveToFile(SaveDlg.FileName);
end;

procedure TBWSectionsPropDlg.EditBtnClick(Sender: TObject);
var SectDlg : TSectEdit;
    THS     : TBWHeaderSection;
        Form : TCustomForm;
begin
  SectDlg:=nil;
  if SecList.ItemIndex=-1 then exit;
  try
    SectDlg:=TSectEdit.Create(self);
    THS:=TBWHeaderSection(SecList.Items.Objects[SecList.ItemIndex]);
    SectDlg.Item:=THS;
    SectDlg.SetData;
    if SectDlg.ShowModal=mrOk then
    begin
      SectDlg.GetData;
      SecList.Items[SecList.ItemIndex]:=THS.Text;
      Form := GetParentForm(DSect.Owner);
      if (Form <> nil) and (Form.Designer <> nil)
         then Form.Designer.Modified;
    end;
  finally
    SectDlg.Free;
  end;
end;

procedure TBWSectionsPropDlg.AddBtnClick(Sender: TObject);
var THS: TBWHeaderSection;
    Form : TCustomForm;
begin
  THS:=DSect.AddSection;
  THS.Text:='New section';
  SecList.Items.AddObject(THS.Text,THS);
  Form := GetParentForm(DSect.Owner);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
  SecListClick(Sender);
end;

procedure TBWSectionsPropDlg.DeleteBtnClick(Sender: TObject);
var THS: TBWHeaderSection;
    Form : TCustomForm;
begin
  if SecList.ItemIndex=-1 then exit;
  THS:=TBWHeaderSection(SecList.Items.Objects[SecList.ItemIndex]);
  SecList.Items.Delete(SecList.ItemIndex);
  DSect.DeleteSection(THS);
  Form := GetParentForm(DSect.Owner);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
  SecListClick(Sender);
end;

procedure TBWSectionsPropDlg.UpBtnClick(Sender: TObject);
var i  : integer;
    THS: TBWHeaderSection;
  Form : TCustomForm;
begin
  if SecList.ItemIndex=-1 then exit;
  i:=SecList.ItemIndex;
  THS:=TBWHeaderSection(SecList.Items.Objects[i]);
  SecList.Items.Exchange(i,i-1);
  DSect.MoveSection(THS,i-1);
  Form := GetParentForm(DSect.Owner);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
end;

procedure TBWSectionsPropDlg.DownBtnClick(Sender: TObject);
var i  : integer;
    THS: TBWHeaderSection;
    Form : TCustomForm;
begin
  if SecList.ItemIndex=-1 then exit;
  i:=SecList.ItemIndex;
  THS:=TBWHeaderSection(SecList.Items.Objects[i]);
  SecList.Items.Exchange(i,i+1);
  DSect.MoveSection(THS,i+1);
  Form := GetParentForm(DSect.Owner);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
end;

procedure TBWSectionsPropDlg.SecListClick(Sender: TObject);
begin
  if (SecList.ItemIndex=0) or (SecList.ItemIndex=-1) then
    UpBtn.Enabled:=false
    else UpBtn.Enabled:=true;
  if SecList.ItemIndex=SecList.Items.Count-1 then
    DownBtn.Enabled:=false
    else DownBtn.Enabled:=true;
  DeleteBtn.Enabled:=SecList.ItemIndex<>-1;
  EditBtn.Enabled:=SecList.ItemIndex<>-1;
end;

procedure TBWSectionsPropDlg.FormCreate(Sender: TObject);
begin
  SecListClick(sender);
end;

procedure TBWSectionsPropDlg.SecListKeyPress(Sender: TObject;
  var Key: Char);
begin
  SecListClick(sender);
end;

end.
